from gaw import GawServer, entrypoint, GawClient

class A(object):

    name = 'A'

    @entrypoint
    def home(self):
        return 'A home'

class B(object):

    name = 'B'

    @entrypoint
    def home(self):
        return 'B home'

class Both(object):

    name = 'Both'

    @entrypoint
    def home(self):
        client = GawClient(ip='localhost', port=12345)
        A = client.A
        B = client.B
        return A.home() + B.home()

GawServer(ip='0.0.0.0', port=12345).add(A).add(B).add(Both).run()


from gawclient import client_class
from test_gaw_with_interfaceclass import Interface

@client_class(ip='127.0.0.1', port=3500)
class Service(Interface): pass

service = Service()
print(service.plus(10, 20))

import time

start_time = time.time()
for i in range(1000):
    service.plus(10, 20)
end_time = time.time()
print('elapsed:', (end_time - start_time), 'ms')


from __future__ import absolute_import
from gaw.postoffice.client import PostofficeClient


client = PostofficeClient(ip='0.0.0.0',
                         port=4444,
                         secret='anfJdufzm2FSbQca0dD8RgN01SO4Pgn0',
                         is_encrypt=True,
                         verbose=True)

from multiprocessing.pool import ThreadPool

p = ThreadPool(100)
p.map(client.send, [i for i in range(100)])


from gaw import Serializable

class MathResult(Serializable):

    def __init__(self, header, result):
        self.header = header
        self.result = result

    def get_result(self):
        return self.result

    def get_header(self):
        return self.header



from __future__ import absolute_import
from gaw import GawClient
from gaw.test_serializable import MathResult # must import

client = GawClient('127.0.0.1', 5555, secret='Qx9XFxN17+zkUdcBIGZ0A1sQTkUSP4SZ', is_encrypt=True, verbose=True)
rpc = client.math_service

# test serializable
print(rpc.plus(a=10, b=20).__dict__)
print(rpc.multiply(10, 20).__dict__)


from __future__ import print_function, absolute_import
from gaw.entrypoint import Entrypoint, entrypoint
from gaw.jsonsocketserver import JsonSocketServer
from functools import wraps
import inspect

INTERFACE_CLASS_ATTR = '__gaw_interface_class__'
INTERFACE_CLASS_METHOD_ATTR = '__gaw_interface_class_method__'

def pluck(d, *args):
    assert isinstance(d, dict)
    return (d[arg] for arg in args)

class GawServer(object):

    def __init__(self, ip, port, secret=None, is_encrypt=False, verbose=False):
        self.ip = ip
        self.port = port
        self.secret = secret
        self.is_encrypt = is_encrypt
        self.verbose = verbose
        self.socketserver = JsonSocketServer(ip, port, secret=secret, is_encrypt=is_encrypt, verbose=verbose)

        self.path = dict()

    def add(self, service_class, *args, **kwargs):
        service_name = service_class.name

        print('gawserver: running service', service_name)

        entry_methods = Entrypoint.get_entrypoints_from_class(service_class)

        for method_name in entry_methods:
            path = '{}/{}'.format(service_name, method_name)

            if self.verbose: print('gawserver: add path {}'.format(path))

            # add path
            self.socketserver.register_route(path, self.router)

            # save misc
            self.path[path] = dict(service_name=service_name,
                                   service_class=service_class,
                                   method_name=method_name,
                                   args=args,
                                   kwargs=kwargs)

        return self

    def run(self):
        self.socketserver.start()

    def router(self, path, **payload):
        if self.verbose:
            print('gawserver: router path:', path)

        cls, method_name, args, kwargs = pluck(self.path[path], 'service_class', 'method_name', 'args', 'kwargs')

        # init a new instance of the class
        cls_instance = cls(*args, **kwargs)

        # note: parsing and serializing the response is now the job of json socket server
        # call the designated method
        args = payload['args']
        kwargs = payload['kwargs']
        result = getattr(cls_instance, method_name)(*args, **kwargs)

        if self.verbose:
            print('gawserver: result ', result)

        return result

def interface_class(cls):
    '''
    wrap a class in which all methods are "entrypoints", and is a template for service client
    '''
    assert hasattr(cls, 'name'), 'intf_cls should have a name defined'
    setattr(cls, INTERFACE_CLASS_ATTR, True)

    for name, obj in inspect.getmembers(cls, predicate=lambda x: inspect.isfunction(x) or inspect.ismethod(x)): # for python 2 and 3 support
        @wraps(obj)
        def wrapper(*args, **kwargs):
            return obj(*args, **kwargs)

        setattr(wrapper, INTERFACE_CLASS_METHOD_ATTR, True)

        setattr(cls, name, wrapper)

    return cls


def service_class(cls):
    '''
    wrap a class in which methods according to intf_cls are entrypoints
    '''

    base = [
        c
        for c in inspect.getmro(cls)
        if c is not cls and hasattr(c, INTERFACE_CLASS_ATTR)
        ]

    assert len(base) == 1, 'should have only one interface_class decorated class'

    intf_cls = base.pop()

    assert hasattr(intf_cls, INTERFACE_CLASS_ATTR), 'intf_cls should be decorated by interface_class'
    assert hasattr(intf_cls, 'name'), 'intf_cls should have a name defined'

    intf_methods = [
        name
        for name, obj in inspect.getmembers(intf_cls)
        if hasattr(obj, INTERFACE_CLASS_METHOD_ATTR)
        ]

    for name in intf_methods:
        method = getattr(cls, name)

        @entrypoint
        @wraps(method)
        def wrapper(*args, **kwargs):
            return method(*args, **kwargs)

        setattr(cls, name, wrapper)

    return cls


from gaw.gawserver import GawServer, interface_class, service_class

@interface_class
class Interface(object):
    name = 'Service'

    def plus(self, a, b): pass

@service_class
class Service(Interface):

    def plus(self, a, b):
        return a + b

if __name__ == '__main__':
    GawServer(ip='0.0.0.0', port=3500).add(Service).run()


from gaw.gawclient import GawClient
from gaw.gawserver import GawServer, interface_class, service_class
from gaw.entrypoint import entrypoint
from gaw.serializable.serializable import Serializable

# exceptions
from gaw.jsonsocketserver.exceptions import JsonSocketException
from gaw.postoffice.exceptions import PostofficeException
from gaw.serializable.exceptions import SerializeError

__version__ = '0.7.0'


from __future__ import absolute_import
from gaw.serializable.serializable import Serializable

class RequestDataType(Serializable):

    def __init__(self,
                 id,
                 path,
                 payload):
        self.id = id
        self.path = path
        self.payload = payload

class ResponseDataType(Serializable):

    def __init__(self,
                 resp_to,
                 success,
                 payload):
        self.resp_to = resp_to
        self.success = success
        self.payload = payload


from gaw.serializable.serializable import Serializable
from datetime import datetime

class A(Serializable):

    def __init__(self, a, b):
        self.a = a
        self.b = b

class B(Serializable):

    def __init__(self, c):
        self.c = c

# test nested object
a = A(10, B(datetime.now()))
da = a.dict()
print(da)
aa = Serializable.parse(da)
print(aa.a, aa.b.c)
print('json:', Serializable.json(a))

# test list
a = [10, ['aoeu', B(10)], 10.10, A(10, 20)]
sa = Serializable.serialize(a)
print(sa)
b = Serializable.parse(sa)
print(b)


# test tuple
a = tuple([10, 'aoeu', 10.10, A(10, 20)])
sa = Serializable.serialize(a)
print(sa)
b = Serializable.parse(sa)
print(b)

# test dict
a = dict(a=10,
         b=20,
         c=dict(
             a=A(10,[1,2,3, B(40)]),
             b=B(30)),
         d=A(30,40))
sa = Serializable.serialize(a)
print(sa)
b = Serializable.parse(sa)
print(b)
print(b['c'])
print(b['c']['a'])
print(b['c']['a'].__dict__)
print(b['c']['b'].__dict__)
print(b['d'].__dict__)

print('json:', Serializable.json(a))

class C(object):

    def __init__(self, a, b):
        self.a = a
        self.b = b

c = C(a=datetime.now(), b=a)
print('json:', Serializable.json(c))

class D(object):

    def __init__(self, a, b):
        self.a = a
        self.b = b

    def __iter__(self):
        for k, v in self.__dict__.items():
            yield k, v

print('json:', Serializable.json(D(10, 20)))